#!/usr/bin/perl

BEGIN
{
        use File::Basename;
        my $dirprg=dirname($0);
        unshift @INC, "$dirprg/";
		$ENV{PATH} .= ":$dirprg";
		$ENV{LANG} = "C";
}

use strict;
use warnings;
use General;
use ParamParser;
use Carp;

my $DEF_MINPCI = 94;
my $DEF_LA4FC  = "LA4Falcon";

MAIN:
{
	my $o_param = New ParamParser('GETOPTLONG',\&Usage,'out=s','la4falcon=s','lof=s','minpci=s','preadsdb=s');

	$o_param->SetUnlessDefined('minpci',$DEF_MINPCI);
	$o_param->SetUnlessDefined('la4falcon',$DEF_LA4FC);
	my $prg_la4falcon = &GetExePath($o_param->Get('la4falcon'));

	$o_param->AssertDefined('out','lof','preadsdb');
	my $minpci   = $o_param->Get('minpci');
	my $preadsdb = $o_param->Get('preadsdb');

	my $f_out = $o_param->GetStreamOut('out');
	my $ra_lof = $o_param->GetListOfFiles('lof');
	foreach my $file (@$ra_lof)
	{
		my $f_in = &GetStreamIn("$prg_la4falcon -m $preadsdb $file|");
		while(<$f_in>)
		{
			@_=split;
			print $f_out $_ if ( $_[3]>=$minpci );
		}
		$f_in->close;
	}
	$f_out->close;
}

sub Usage
{
print STDERR<<END
Collect filtered overlaps in falcon format (input of til-r software)

Usage:

$0 --lof '1-preads_ovl/m*/preads*.las' --preadsdb 1-preads_ovl/preads.db --out 1-preads_ovl.i94.ovl.gz 

Requirement: 
- the outputs files of a run of falcon (from preads) must be available
- falcon environment must be initialized ($DEF_LA4FC must be in the PATH)

	--help      this message
Mandatory
	--lof       'quoted string'   pattern corresponding to the whole list of the paths of las files (e.g '1-preads_ovl/m*/preads*.las')
	--preadsdb  string            filename of the index las files (e.g 1-preads_ovl/preads.db)
	--out       string            output file (list of overlaps in falcon format)

Optional
	--minpci    integer           [$DEF_MINPCI]
	--la4falcon string            [$DEF_LA4FC]

END
}
